<?php
/**
 * Created by PhpStorm.
 * User: jderosa
 * Date: 28/10/19
 * Time: 16.55
 */

namespace FiloBlu\Rma\Model;
use FiloBlu\Rma\Api\ChangeSizeInterface;
use FiloBlu\Rma\Helper\ChangeSizeAttributeHelper;
use FiloBlu\Rma\Logger\Logger;
use Magento\InventorySalesApi\Api\Data\SalesChannelInterface;
use Magento\Sales\Model\Order\ItemRepository;
use Magento\Catalog\Model\ProductRepository;
use Magento\ConfigurableProduct\Api\LinkManagementInterface;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Catalog\Model\ResourceModel\Product\CollectionFactory;
use Magento\CatalogInventory\Api\StockStateInterface;
use Magento\Customer\Model\Session;
use Magento\Catalog\Model\ResourceModel\Product;
use FiloBlu\Rma\Helper\RmaHelper;
use Magento\ConfigurableProduct\Model\ResourceModel\Product\Type\Configurable;
use Magento\Framework\App\Helper\Context;
use Magento\Sales\Model\OrderRepository;
use Magento\Framework\Module\Manager;

class ChangeSize implements ChangeSizeInterface
{

    const STOCK_MIN_QTY = 'filoblu_disablerma_section/rma_group/rma_minimum_qty';

    /**
     * @var ChangeSizeAttributeHelper
     */
    protected $changeSizeAttributeHelper;

    /**
     * @var ItemRepository
     */
    protected $itemRepository;

    /**
     * @var ProductRepository
     */
    protected $productRepository;

    /**
     * @var LinkManagementInterface
     */
    protected $linkManagment;

    /**
     * @var StoreManagerInterface
     */
    protected $storeManager;

    /**
     * @var JsonFactory
     */
    protected $jsonFactory;
    /**
     * @var CollectionFactory
     */
    protected $collectionFactory;

    /**
     * @var StockStateInterface
     */
    protected $stockItem;

    /**
     * @var Session
     */
    protected $customerSession;

    /**
     * @var Logger
     */
    protected $logger;

    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var \Magento\Catalog\Model\ResourceModel\Product
     */
    protected $resourceModel;

    /**
     * @var Configurable
     */
    protected $configurableResourceModel;

    /**
     * @var RmaHelper
     */
    protected $rmaHelper;

    /**
     * @var Context
     */
    protected $context;
    /**
     * @var ObjectManager
     */
    private $objectManager;
    /**
     * @var Manager
     */
    private $manager;

    /**
     * ChangeSize constructor.
     * @param ChangeSizeAttributeHelper $changeSizeAttributeHelper
     * @param ItemRepository $itemRepository
     * @param ProductRepository $productRepository
     * @param LinkManagementInterface $linkManagement
     * @param JsonFactory $jsonFactory
     * @param CollectionFactory $collectionFactory
     * @param StockStateInterface $stockItem
     * @param Session $customerSession
     * @param Product $resourceModel
     * @param Configurable $configurableResourceModel
     * @param RmaHelper $rmaHelper
     * @param Context $context
     * @param StoreManagerInterface $storeManager
     * @param Logger $logger
     * @param Manager $manager
     */
    public function __construct(
        ChangeSizeAttributeHelper $changeSizeAttributeHelper,
        ItemRepository $itemRepository,
        ProductRepository $productRepository,
        LinkManagementInterface $linkManagement,
        JsonFactory $jsonFactory,
        CollectionFactory $collectionFactory,
        StockStateInterface $stockItem,
        Session $customerSession,
        Product $resourceModel,
        Configurable $configurableResourceModel,
        RmaHelper $rmaHelper,
        Context $context,
        StoreManagerInterface $storeManager,
        Logger $logger,
        Manager $manager
    )
    {
        $this->changeSizeAttributeHelper =$changeSizeAttributeHelper;
        $this->context = $context;
        $this->itemRepository =$itemRepository;
        $this->productRepository = $productRepository;
        $this->linkManagment = $linkManagement;
        $this->jsonFactory = $jsonFactory;
        $this->collectionFactory = $collectionFactory;
        $this->stockItem = $stockItem;
        $this->customerSession =$customerSession;
        $this->scopeConfig = $context->getScopeConfig();
        $this->resourceModel = $resourceModel;
        $this->configurableResourceModel = $configurableResourceModel;
        $this->rmaHelper = $rmaHelper;
        $this->storeManager =$storeManager;
        $this->objectManager = ObjectManager::getInstance();
        $this->logger = $logger;
        $this->manager = $manager;
    }


    /**
     * @return mixed|string
     */
    public function getLabel()
    {
        try {
            return $this->changeSizeAttributeHelper->getChangeSizeAttributeOptionId();
        }catch (\Exception $e){
            $this->logger->critical($e->getMessage());
            return ChangeSizeAttributeHelper::REASON_CHANGE_SIZE_LABEL;
        }
    }


    /**
     * @param string $productIds
     * @param int $websiteId
     * @return array|mixed
     */
    public function getChangeSizeEnabledProducts(string $productIds, int $websiteId){
        try {
            $enabledProducts = [];
            if($this->changeSizeAttributeHelper->isChangeSizeEnabled($websiteId)) {

                $productIds = explode(",", $productIds);
                foreach ($productIds as $productId) {
                    $item = $this->itemRepository->get($productId);
                    $storeId = $this->storeManager->getWebsite($websiteId)->getDefaultStore()->getId();
                    $product = $this->productRepository->get($item->getSku(), false, $storeId);
                    $parent_products = $this->configurableResourceModel->getParentIdsByChild($product->getId());
                    if ($parent_products) {
                        $enabledProducts[] = $productId;
                    }
                }
            }
            return $enabledProducts;
        }catch (\Exception $e){
            $this->logger->critical($e->getMessage());
            return [];
        }

    }


    /**
     * @param int $itemId
     * @param int $qty
     * @param int $resolution
     * @param int $reason
     * @param int $websiteId
     * @return array|mixed
     */
    public function getSizes(int $itemId,int $qty,int $resolution, int $reason, int $websiteId)
    {

        //check if reason is allowed for ChangeSize Order
        if($this->rmaHelper->checkIfAllowedMagentoChangeSizeReason($reason)) {
            try {
                $storeId = $this->storeManager->getWebsite($websiteId)->getDefaultStore()->getId();
                $size = [];
                if ($resolution == $this->changeSizeAttributeHelper->getChangeSizeAttributeOptionId()) {
                    if($qty == 0){
                        $qty = 1;
                    }
                    $item = $this->itemRepository->get($itemId);
                    $product = $this->productRepository->get($item->getSku(), false, $storeId);
                    $parent_products = $this->configurableResourceModel->getParentIdsByChild($product->getId());
                    if(!$parent_products){
                        $size = array(["value" => -1, "text" => "No Configurable Products", "selected" => false]);
                        return $size;
                    }
                    $parents_sku = $this->resourceModel->getProductsSku($parent_products);
                    $childProducts = $this->linkManagment->getChildren($parents_sku[0]['sku']);

                    foreach ($childProducts as $child) {
                        if ($this->isSaleable($child, $storeId)) {
                            if($child->getId() === null){
                                $child->setId($child->getIdBySku($child->getSku()));
                            }
                            if ($this->getProductQtyById($child->getId(),$storeId) >= $this->getMinQty() + $qty && $product->getId() != $child->getId()) {
                                $size[] = ["value" => $child->getId(), "text" => $child->getResource()->getAttribute('size')->getFrontend()->getValue($child), "selected" => false];
                            }
                        }
                    }
                    if (count($size) == 0) {
                        $size = array(["value" => 0, "text" => "No Available Size", "selected" => true]);
                    }
                }
                return $size;
            } catch (\Exception $e) {
                $this->logger->critical($e->getMessage());
                return [];
            }
        }else{
            return  array(["value" => 0, "text" => "The Reason Selected is not compatible with Change Size Resolution.", "selected" => true]);
        }

    }

    /**
     * @param $product
     * @param int $storeId
     * @return bool
     */
    public function isSaleable($product, int $storeId)
    {
        try{
            $websiteId = $this->storeManager->getStore($storeId)->getWebsiteId();
            $products = $this->collectionFactory->create()
                ->setStoreId($storeId)
                ->addPriceData($this->customerSession->getCustomerGroupId(),$websiteId)
                ->addAttributeToFilter('sku',['eq'=>$product->getSku()])
                ->getItems();
            return $products ? true : false;
        }catch (\Exception $e){
            $this->logger->critical($e->getMessage());
            return false;
        }catch (\Throwable $t){
            $this->logger->critical($t->getMessage());
            return false;
        }
    }


    /**
     * @param $productId
     * @param $storeId
     * @return float|int
     */
    public function getProductQtyById($productId,$storeId)
    {
        try{
            if(!interface_exists('Magento\InventorySalesApi\Api\GetProductSalableQtyInterface') || !$this->manager->isEnabled('Magento_InventorySalesApi')){
                return $this->stockItem->getStockQty($productId);
            }
            $websiteCode = $this->storeManager->getWebsite($this->storeManager->getStore($storeId)->getWebsiteId())->getCode();
            $getProductSalableQty = $this->objectManager->create(\Magento\InventorySalesApi\Api\GetProductSalableQtyInterface::class);
            $stockResolver = $this->objectManager->create(\Magento\InventorySalesApi\Api\StockResolverInterface::class);
            $stockId = $stockResolver->execute(SalesChannelInterface::TYPE_WEBSITE, $websiteCode)->getStockId();
            $product = $this->productRepository->getById($productId);
            return (int)$getProductSalableQty->execute($product->getSku(), $stockId);
        }catch (\Exception $e){
            $this->logger->critical($e->getMessage());
            return 0;
        }catch (\Throwable $t){
            $this->logger->critical($t->getMessage());
            return 0;
        }

    }

    /**
     * @return mixed
     */
    public function getMinQty()
    {
        return (int) $this->scopeConfig->getValue(self::STOCK_MIN_QTY);
    }


}


